@@ -36,14 +36,13 @@ getEffects<- function(x, nintn = 10)
             rateEffects <- c(rateEffects,
                              paste(symmetricRateEffects[-(1:2), 1], varname))
             rateFunctions <- c(rateFunctions, symmetricRateEffects[-(1:2), 2])
-            rateShortNames <- symmetricRateEffects[, 3]
             ratePeriods <- c(ratePeriods, rep(NA, nrow(symmetricRateEffects)-2))
             rateTypes <- c(rateTypes, rep('structural',
                                           nrow(symmetricRateEffects)-2))
             objEffects <- symmetricObjEffects[, 1]
             objFunctions <- symmetricObjEffects[, 2]
             objEndowment <- symmetricObjEffects[, 3]
-            objShortnames <- symmetricObjEffects[, 4]
+            objShortNames <- symmetricObjEffects[, 4]
             objParms <- symmetricObjEffects[, 5]
             objEffects <- createObjEffectList(objEffects, objFunctions,
                                               objEndowment, objShortNames,
@@ -720,8 +719,8 @@ getEffects<- function(x, nintn = 10)
     n <- length(xx$depvars)
     types <- sapply(xx$depvars, function(x)attr(x, 'type'))
     sparses <- sapply(xx$depvars, function(x)attr(x, 'sparse'))
-    if (any(sparses))
-        require(Matrix)
+   ## if (any(sparses))
+   ##     require(Matrix)
     nOneModes <- sum(types == 'oneMode')
     nBehaviors <- sum(types == 'behavior')
     effects <- vector('list',n)
@@ -1006,6 +1005,8 @@ getNetworkStartingVals <- function(depvar, structValid=TRUE)
                 mymat1@x[use] <- mymat1@x[use] - 10
                 use <- mymat2@x %in% c(10, 11)
                 mymat2@x[use] <- mymat2@x[use] - 10
+                mymat1 <- drop0(mymat1)
+                mymat2 <- drop0(mymat2)
             }
             else
             {
@@ -1016,8 +1017,8 @@ getNetworkStartingVals <- function(depvar, structValid=TRUE)
             }
             diag(mymat1) <- NA
             diag(mymat2) <- NA
-            mydif <- mymat2-mymat1
-            matdiff[i] <-sum(abs(mydif), na.rm=TRUE)
+            mydif <- mymat2 - mymat1
+            matdiff[i] <- sum(abs(mydif), na.rm=TRUE)
             tmp <- table(mydif@x)
             tmp00 <- nactors * nactors - length(mydif@x)
             tmp <- c(tmp00, tmp[c(3, 1, 2)])
@@ -1030,7 +1031,7 @@ getNetworkStartingVals <- function(depvar, structValid=TRUE)
     }
     distance <- attr(depvar, "distance" )
     if (attr(depvar,'symmetric'))
-        startRate<- nactors * (0.2 + distance)/(tmp['matcnt',]+1)
+        startRate<- nactors * (0.2 + distance)/(tmp['matcnt',] %/% 2 +1)
     else
         startRate<- nactors * (0.2 + 2 * distance)/(tmp['matcnt',]+1)
     startRate <- pmax(0.1, startRate)
@@ -1040,7 +1041,7 @@ getNetworkStartingVals <- function(depvar, structValid=TRUE)
     if (attr(depvar,'symmetric'))
     {
         matchange <- matchange %/% 2
-        matcnt <- matcnt %/% 2
+       ## matcnt <- matcnt %/% 2
     }
     p01 <- ifelse (matchange[1,] + matchange[2,] >=1,
                    matchange[2,]/(matchange[1,]+matchange[2,]),0.5)
