@@ -12,6 +12,7 @@
 imagepath <- ''
 ##csvpath <- ''
 ## or .onAttach?
+pkgpath <- ''
 .onLoad <- function(libname,pkgname){
 #   dllpath <<- if (nzchar(.Platform$r_arch))
 ##       file.path(libname, pkgname, "libs", .Platform$r_arch,
@@ -21,12 +22,13 @@ imagepath <- ''
  #                paste('RSiena', .Platform$dynlib.ext, sep=''))
 #  #  data('sysdata',package='RSiena')
    imagepath <<- file.path(libname, pkgname, paste('ilcampo.gif'))
+   pkgpath<<- file.path(libname, pkgname)
+##   cat(pkgpath,'\n')
  ##  csvpath<<- file.path(libname,pkgname)
  #   library.dynam("RSiena",package=pkgname)
  #  cat (libname,pkgname,'\n')
 }
 
-## not used yet.
 .onUnload <- function(libpath) {
     library.dynam.unload("RSiena", libpath)
 }
@@ -35,3 +37,5 @@ imagepath <- ''
 #{
 #    cat(libpath,'\n')
 #}
+
+
