@@ -22,14 +22,20 @@
 #include "CovariateAlterEffect.h"
 #include "CovariateEgoAlterEffect.h"
 #include "CovariateEgoEffect.h"
+#include "CovariateIndirectTiesEffect.h"
 #include "CovariateSimilarityEffect.h"
+#include "DenseTriadsEffect.h"
 #include "DensityEffect.h"
+#include "DistanceTwoEffect.h"
 #include "DyadicCovariateMainEffect.h"
+#include "DyadicCovariateReciprocityEffect.h"
 #include "IndegreeEffect.h"
 #include "IndegreeActivityEffect.h"
 #include "IndegreePopularityEffect.h"
 #include "InInDegreeAssortativityEffect.h"
 #include "InOutDegreeAssortativityEffect.h"
+#include "InverseOutdegreeEffect.h"
+#include "InverseSquaredOutdegreeEffect.h"
 #include "LinearShapeEffect.h"
 #include "OutdegreeEffect.h"
 #include "OutdegreeActivityEffect.h"
@@ -46,5 +52,8 @@
 #include "TransitiveTiesEffect.h"
 #include "TransitiveTriadsEffect.h"
 #include "TransitiveTripletsEffect.h"
+#include "WWXClosureEffect.h"
+#include "WXXClosureEffect.h"
+#include "XWXClosureEffect.h"
 
 #endif /*ALLEFFECTS_H_*/
