@@ -1,10 +1,10 @@
 /******************************************************************************
  * SIENA: Simulation Investigation for Empirical Network Analysis
- * 
+ *
  * Web: http://www.stats.ox.ac.uk/~snijders/siena/
- * 
+ *
  * File: CovariateSimilarityEffect.h
- * 
+ *
  * Description: This file contains the definition of the
  * CovariateSimilarityEffect class.
  *****************************************************************************/
@@ -18,17 +18,26 @@ namespace siena
 {
 
 /**
- * Covariate-related similarity effect (see manual).
+ * Covariate-related similarity and covariate-related similarity x reciprocity
+ * effects (see manual).
  */
 class CovariateSimilarityEffect : public CovariateDependentNetworkEffect
 {
 public:
-	CovariateSimilarityEffect(const EffectInfo * pEffectInfo);
-	
+	CovariateSimilarityEffect(const EffectInfo * pEffectInfo,
+		bool reciprocal);
+
 	virtual double calculateTieFlipContribution(int alter) const;
-	virtual double evaluationStatistic(Network * pNetwork) const;
-	virtual double endowmentStatistic(Network * pInitialNetwork,
-		Network * pLostTieNetwork) const;
+
+protected:
+	virtual double statistic(Network * pNetwork,
+		Network * pSummationTieNetwork) const;
+
+private:
+	// Indicates if the reciprocal version of the similarity effect is
+	// required
+
+	bool lreciprocal;
 };
 
 }
